"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServer = void 0;
const buffer_1 = require("buffer");
const process_1 = __importDefault(require("process"));
const express_1 = __importDefault(require("express"));
const get_port_1 = __importDefault(require("get-port"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const config_js_1 = require("./config.js");
const app = (0, express_1.default)();
let consecutiveHealthFailures = 0;
const createServer = async ({ config, serverPort, targetPort }) => {
    const port = await (0, get_port_1.default)({ port: serverPort });
    app.get('/up', (req, res) => {
        res.status(200).send({ status: 'ok' });
    });
    app.get('/health', async (req, res) => {
        var _a;
        const healthPath = process_1.default.env.DEV_SERVER_HEALTH_PATH || '/';
        try {
            const response = await (0, node_fetch_1.default)(`http://localhost:${targetPort}${healthPath}`, {
                timeout: config_js_1.HEALTH_TIMEOUT,
                method: 'HEAD',
            });
            const status = response.status >= 200 && response.status < 500 ? 200 : 500;
            if (status === 200) {
                consecutiveHealthFailures = 0;
            }
            else {
                consecutiveHealthFailures += 1;
            }
            res.status(status).send({ status: 'got response', failures: consecutiveHealthFailures });
        }
        catch (error) {
            consecutiveHealthFailures += 1;
            if ((_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.status) {
                res
                    .status(error.response.status)
                    .send({ status: 'error with status code', failures: consecutiveHealthFailures });
            }
            else {
                res.status(500).send({ status: 'error', error, failures: consecutiveHealthFailures });
            }
        }
        if (consecutiveHealthFailures >= 3) {
            console.error(`Failed to reach dev server at path "${healthPath}" after ${consecutiveHealthFailures} attempts. Restarting...`);
        }
        else if (consecutiveHealthFailures === 2) {
            console.error(`Failed to reach dev server at path "${healthPath}" after multiple attempts. Retrying...`);
        }
    });
    app.post('/content-refresh*', express_1.default.raw({ type: '*/*', limit: '10mb' }), async (req, res) => {
        var _a, _b;
        let urlSuffix = (_b = (_a = req.params) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : '';
        if (urlSuffix.startsWith('/')) {
            urlSuffix = urlSuffix.slice(1);
        }
        let endpoint = '__refresh';
        if ((urlSuffix === null || urlSuffix === void 0 ? void 0 : urlSuffix.length) > 0) {
            endpoint += `/${encodeURIComponent(urlSuffix)}`;
        }
        let { body } = req;
        if (body instanceof buffer_1.Buffer) {
            body = body.toString();
        }
        else if (typeof body === 'object' && Object.keys(body).length === 0) {
            body = undefined;
        }
        const url = `http://localhost:${targetPort}/${endpoint}`;
        await (0, node_fetch_1.default)(url, {
            method: 'POST',
            headers: {
                'Content-Type': req.get('Content-Type') || 'application/json',
            },
            body,
        });
        res.status(200).send({ status: 'ok' });
    });
    app.post('/git/pull', express_1.default.json(), async (req, res) => {
        var _a;
        try {
            const { stdout, stderr, exitCode } = await config.utils.run('git', [
                'pull',
                '--rebase',
                '--autostash',
                '-Xtheirs',
                'origin',
                ((_a = req.body) === null || _a === void 0 ? void 0 : _a.branch) || '',
            ]);
            if (exitCode === 0) {
                res.status(200).send({ status: 'ok', stdout });
            }
            else {
                res.status(500).send({ status: 'error', error: stderr, stdout, exitCode });
            }
        }
        catch (error) {
            console.log(error);
            res.status(500).send({ status: 'error', error });
        }
    });
    return new Promise((resolve) => {
        app.listen(port, () => {
            console.log('[dev-server-plugin] listening on port', port);
            resolve(port);
        });
    });
};
exports.createServer = createServer;
if (require.main === module) {
    (0, exports.createServer)({ serverPort: config_js_1.DEFAULT_SERVER_PORT, targetPort: config_js_1.DEFAULT_TARGET_PORT });
}
